@@ -1,42 +1,21 @@
-# coding=utf-8
-
-from __future__ import print_function
-from __future__ import unicode_literals
-
+import functools
+import importlib.resources
 import json
 import os
 import shutil
 import signal
 import socket
+import subprocess
 import sys
 import tempfile
-import subprocess
 import threading
+from typing import Optional
 
-try:
-    from builtins import next
-    from builtins import object
-    from builtins import open
-    from builtins import str
-except ImportError:
-    pass
-
-from .error import (VarlinkError, InterfaceNotFound, VarlinkEncoder, BrokenPipeError)
-from .scanner import (Interface, _Method)
-
-PY2 = sys.version_info[0] == 2
-PY3 = (sys.version_info[0] >= 3)
-
-if PY2:
-    FileNotFoundError = IOError
-    ChildProcessError = OSError
+from .error import InterfaceNotFound, VarlinkEncoder, VarlinkError
+from .scanner import Interface, _Method
 
 
-class ConnectionError(OSError):
-    pass
-
-
-class ClientInterfaceHandler(object):
+class ClientInterfaceHandler:
     """Base class for varlink client, which wraps varlink methods of an interface to the class"""
 
     def __init__(self, interface, namespaced=False):
@@ -62,11 +41,11 @@ class ClientInterfaceHandler(object):
             if isinstance(member, _Method):
                 self._add_method(member)
 
-    def close(self):
+    def close(self) -> None:
         """To be implemented."""
         raise NotImplementedError
 
-    def _send_message(self, out):
+    def _send_message(self, out) -> None:
         """To be implemented.
 
         This should send a varlink message to the varlink service adding a trailing zero byte.
@@ -81,17 +60,13 @@ class ClientInterfaceHandler(object):
         raise NotImplementedError
 
     def _add_method(self, method):
-
         def _wrapped(*args, **kwargs):
             if "_more" in kwargs and kwargs.pop("_more"):
                 return self._call_more(method.name, *args, **kwargs)
             else:
                 return self._call(method.name, *args, **kwargs)
 
-        if sys.version_info.major >= 3:
-            _wrapped.__name__ = method.name
-        else:
-            _wrapped.__name__ = method.name.encode("latin-1")
+        _wrapped.__name__ = method.name
 
         # FIXME: add comments
         if method.signature:
@@ -107,15 +82,15 @@ class ClientInterfaceHandler(object):
         message = next(self._next_message())
 
         message = json.loads(message)
-        if not 'parameters' in message:
-            message['parameters'] = {}
+        if "parameters" not in message:
+            message["parameters"] = {}
 
-        if 'error' in message and message["error"] != None:
+        if "error" in message and message["error"] is not None:
             self._in_use = False
-            e = VarlinkError.new(message, self._namespaced)
+            e = VarlinkError.new(message)
             raise e
         else:
-            return message['parameters'], ('continues' in message) and message['continues']
+            return message["parameters"], ("continues" in message) and message["continues"]
 
     def _call(self, method_name, *args, **kwargs):
         if self._in_use:
@@ -129,15 +104,15 @@ class ClientInterfaceHandler(object):
 
         parameters = self._interface.filter_params("client.call", method.in_type, False, args, kwargs)
 
-        out = {'method': self._interface.name + "." + method_name}
+        out = {"method": self._interface.name + "." + method_name}
 
         if oneway:
-            out['oneway'] = True
+            out["oneway"] = True
 
         if parameters:
-            out['parameters'] = parameters
+            out["parameters"] = parameters
 
-        self._send_message(json.dumps(out, cls=VarlinkEncoder).encode('utf-8'))
+        self._send_message(json.dumps(out, cls=VarlinkEncoder).encode("utf-8"))
 
         if oneway:
             return None
@@ -151,7 +126,9 @@ class ClientInterfaceHandler(object):
         self._in_use = False
 
         if message:
-            message = self._interface.filter_params("client.reply", method.out_type, self._namespaced, message, None)
+            message = self._interface.filter_params(
+                "client.reply", method.out_type, self._namespaced, message, None
+            )
 
         return message
 
@@ -162,17 +139,18 @@ class ClientInterfaceHandler(object):
         method = self._interface.get_method(method_name)
 
         parameters = self._interface.filter_params("client.call", method.in_type, False, args, kwargs)
-        out = {'method': self._interface.name + "." + method_name, 'more': True, 'parameters': parameters}
+        out = {"method": self._interface.name + "." + method_name, "more": True, "parameters": parameters}
 
-        self._send_message(json.dumps(out, cls=VarlinkEncoder).encode('utf-8'))
+        self._send_message(json.dumps(out, cls=VarlinkEncoder).encode("utf-8"))
 
         more = True
         self._in_use = True
         while more:
             (message, more) = self._next_varlink_message()
             if message:
-                message = self._interface.filter_params("client.reply", method.out_type, self._namespaced, message,
-                                                        None)
+                message = self._interface.filter_params(
+                    "client.reply", method.out_type, self._namespaced, message, None
+                )
             yield message
         self._in_use = False
 
@@ -196,31 +174,31 @@ class SimpleClientInterfaceHandler(ClientInterfaceHandler):
         ClientInterfaceHandler.__init__(self, interface, namespaced=namespaced)
         self._connection = file_or_socket
 
-        if hasattr(self._connection, 'send_bytes'):
+        if hasattr(self._connection, "send_bytes"):
             self._send_bytes = True
             self._sendall = False
-        elif hasattr(self._connection, 'sendall'):
+        elif hasattr(self._connection, "sendall"):
             self._send_bytes = False
             self._sendall = True
         else:
-            if not hasattr(self._connection, 'write'):
+            if not hasattr(self._connection, "write"):
                 raise TypeError
             self._sendall = False
             self._send_bytes = False
 
-        if hasattr(self._connection, 'recv_bytes'):
+        if hasattr(self._connection, "recv_bytes"):
             self._recv_bytes = True
             self._recv = False
-        elif hasattr(self._connection, 'recv'):
+        elif hasattr(self._connection, "recv"):
             self._recv = True
             self._recv_bytes = False
         else:
-            if not hasattr(self._connection, 'read'):
+            if not hasattr(self._connection, "read"):
                 raise TypeError
             self._recv = False
             self._recv_bytes = False
 
-        self._in_buffer = b''
+        self._in_buffer = b""
 
     def __enter__(self):
         return self
@@ -228,33 +206,33 @@ class SimpleClientInterfaceHandler(ClientInterfaceHandler):
     def __exit__(self, _type, _value, _traceback):
         self.close()
 
-    def close(self):
+    def close(self) -> None:
         try:
-            if hasattr(self._connection, 'shutdown'):
+            if hasattr(self._connection, "shutdown"):
                 self._connection.shutdown(socket.SHUT_RDWR)
-        except:
+        except Exception:  # TODO: maybe just OSError?
             pass
 
         self._connection.close()
 
-    def _send_message(self, out):
+    def _send_message(self, out) -> None:
         if self._send_bytes:
-            self._connection.send_bytes(out + b'\0')
+            self._connection.send_bytes(out + b"\0")
         elif self._sendall:
-            self._connection.sendall(out + b'\0')
-        elif hasattr:
-            self._connection.write(out + b'\0')
+            self._connection.sendall(out + b"\0")
+        else:
+            self._connection.write(out + b"\0")
 
     def _next_message(self):
         while True:
-            message, sep, self._in_buffer = self._in_buffer.partition(b'\0')
+            message, sep, self._in_buffer = self._in_buffer.partition(b"\0")
             if not sep:
                 # No zero byte found
                 self._in_buffer = message
                 message = None
 
             if message:
-                yield message.decode('utf-8')
+                yield message.decode("utf-8")
                 continue
 
             if self._recv_bytes:
@@ -270,15 +248,8 @@ class SimpleClientInterfaceHandler(ClientInterfaceHandler):
 
 
 def pipe_bridge(reader, writer):
-    if hasattr(reader, "recv"):
-        recv = True
-    else:
-        recv = False
-
-    if hasattr(writer, "sendall"):
-        sendall = True
-    else:
-        sendall = False
+    recv = hasattr(reader, "recv")
+    sendall = hasattr(writer, "sendall")
 
     while True:
         try:
@@ -286,7 +257,7 @@ def pipe_bridge(reader, writer):
                 data = reader.recv(8192)
             else:
                 data = reader.read(1)
-        except:
+        except Exception:  # TODO: maybe just OsError?
             data = []
 
         if len(data) == 0:
@@ -297,16 +268,16 @@ def pipe_bridge(reader, writer):
         try:
             if sendall:
                 writer.sendall(data)
-            elif hasattr:
+            else:
                 writer.write(data)
                 writer.flush()
-        except Exception as e:
+        except Exception:  # TODO: maybe just OsError?
             reader.close()
             writer.close()
             return
 
 
-class Client(object):
+class Client:
     """Varlink client class.
 
         >>> with varlink.Client("unix:/run/org.example.ping") as client, client.open('org.example.ping') as connection:
@@ -342,7 +313,7 @@ class Client(object):
 
         >>> for m in connection.Monitor(_more=True):
         >>>     for e in m.entries:
-        >>>         print("%s: %s" % (e.time, e.message))
+        >>>         print(f"{e.time}: {e.message}")
         2018-01-29 12:19:59Z: [system] Activating via systemd: service name='[…]
         2018-01-29 12:19:59Z: Starting Network Manager Script Dispatcher Service...
         2018-01-29 12:19:59Z: bound to 10.200.159.150 -- renewal in 1423 seconds.
@@ -355,9 +326,17 @@ class Client(object):
     not return a normal namespace wrapped varlink return value, but a generator,
     which yields the return values and waits (blocks) for the service to return more return values
     in the generator's .__next__() call.
-    """
+    """  # noqa: E501
+
     handler = SimpleClientInterfaceHandler
 
+    @classmethod
+    @functools.lru_cache(maxsize=1)
+    def _get_base_interface(cls):
+        return Interface(
+            importlib.resources.files("varlink").joinpath("org.varlink.service.varlink").read_text()
+        )
+
     def __init__(self, address=None, resolve_interface=None, resolver=None):
         """Creates a Client object to reach the interfaces of a varlink service.
         For more constructors see the class constructor methods new_with_*() returning an Client object.
@@ -375,9 +354,7 @@ class Client(object):
         self._child_pid = 0
         self._str = "Client<uninitialized>"
 
-        with open(os.path.join(os.path.dirname(__file__), 'org.varlink.service.varlink')) as f:
-            interface = Interface(f.read())
-            self.add_interface(interface)
+        self.add_interface(self._get_base_interface())
 
         if resolve_interface:
             self._with_interface(resolve_interface, resolver)
@@ -407,8 +384,9 @@ class Client(object):
     def _with_activate(self, argv):
         s = socket.socket(socket.AF_UNIX)
         s.setblocking(False)
+        # TODO: this is not cleaned up
         self._tmpdir = tempfile.mkdtemp()
-        address = self._tmpdir + "/" + str(os.getpid())
+        address = f"{self._tmpdir}/{os.getpid()}"
         s.bind(address)
         s.listen(100)
 
@@ -427,16 +405,17 @@ class Client(object):
                     pass
 
             os.dup2(n, 3)
-            address = address.replace('\0', '@', 1)
+            address = address.replace("\0", "@", 1)
 
             for i in range(1, len(argv)):
-                argv[i] = argv[i].replace("$VARLINK_ADDRESS", "unix:" + address)
+                argv[i] = argv[i].replace("$VARLINK_ADDRESS", f"unix:{address}")
 
-            os.environ["VARLINK_ADDRESS"] = "unix:" + address
+            os.environ["VARLINK_ADDRESS"] = f"unix:{address}"
             os.environ["LISTEN_FDS"] = "1"
             os.environ["LISTEN_FDNAMES"] = "varlink"
             os.environ["LISTEN_PID"] = str(os.getpid())
             os.execvp(argv[0], argv)
+            # TODO: this is unreachable
             sys.exit(1)
         # parent
         s.close()
@@ -465,10 +444,11 @@ class Client(object):
             self._child_pid = p.pid
             return sp[0]
 
-        def new_bridge_socket_compat():
+        def new_bridge_socket_compat() -> socket.socket:
             sp = socket.socketpair()
-            p = subprocess.Popen(" ".join(argv), shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
-                                 close_fds=True)
+            p = subprocess.Popen(
+                " ".join(argv), shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, close_fds=True
+            )
             self._child_pid = p.pid
 
             thread1 = threading.Thread(daemon=True, target=pipe_bridge, args=(sp[1], p.stdin))
@@ -477,8 +457,8 @@ class Client(object):
             thread2.start()
             return sp[0]
 
-        self._str = "Bridge with: '%s'" % " ".join(argv)
-        if sys.platform == 'win32':
+        self._str = f"Bridge with: '{' '.join(argv)}'"
+        if sys.platform == "win32":
             self._socket_fn = new_bridge_socket_compat
         else:
             self._socket_fn = new_bridge_socket
@@ -500,11 +480,11 @@ class Client(object):
         if address.startswith("unix:"):
             self._str = address
             address = address[5:]
-            mode = address.find(';')
+            mode = address.find(";")
             if mode != -1:
                 address = address[:mode]
-            if address[0] == '@':
-                address = address.replace('@', '\0', 1)
+            if address[0] == "@":
+                address = address.replace("@", "\0", 1)
 
             def open_unix():
                 s = socket.socket(socket.AF_UNIX)
@@ -517,14 +497,14 @@ class Client(object):
         elif address.startswith("tcp:"):
             self._str = address
             address = address[4:]
-            p = address.rfind(':')
+            p = address.rfind(":")
             if p != -1:
-                port = address[p + 1:]
+                port = address[p + 1 :]
                 address = address[:p]
             else:
-                raise ConnectionError("Invalid address 'tcp:%s'" % address)
-            address = address.replace('[', '')
-            address = address.replace(']', '')
+                raise ConnectionError(f"Invalid address 'tcp:{address}'")
+            address = address.replace("[", "")
+            address = address.replace("]", "")
 
             def open_tcp():
                 s = socket.create_connection((address, int(port)))
@@ -534,7 +514,7 @@ class Client(object):
             self._socket_fn = open_tcp
 
         elif address is not None:
-            raise ConnectionError("Invalid address '%s'" % address)
+            raise ConnectionError(f"Invalid address '{address}'")
 
         return self
 
@@ -554,19 +534,21 @@ class Client(object):
         if not resolver_address:
             resolver_address = "unix:/run/org.varlink.resolver"
 
-        if interface == 'org.varlink.resolver':
+        if interface == "org.varlink.resolver":
             self._with_address(resolver_address)
         else:
-            with Client.new_with_address(resolver_address) as client, \
-                    client.open('org.varlink.resolver') as _rc:
+            with (
+                Client.new_with_address(resolver_address) as client,
+                client.open("org.varlink.resolver") as _rc,
+            ):
                 # noinspection PyUnresolvedReferences
                 _r = _rc.Resolve(interface)
-                self._with_address(_r['address'])
+                self._with_address(_r["address"])
 
         return self
 
-    def cleanup(self):
-        if hasattr(self, "_tmpdir") and self._tmpdir != None:
+    def cleanup(self) -> None:
+        if hasattr(self, "_tmpdir") and self._tmpdir is not None:
             try:
                 shutil.rmtree(self._tmpdir)
             except FileNotFoundError:
@@ -579,10 +561,12 @@ class Client(object):
                 pass
             try:
                 os.waitpid(self._child_pid, 0)
-            except:
+            except Exception:  # TODO: maybe just ChildProcessError?
                 pass
 
-    def open(self, interface_name, namespaced=False, connection=None):
+    def open(
+        self, interface_name: str, namespaced: bool = False, connection: Optional[socket.socket] = None
+    ) -> SimpleClientInterfaceHandler:
         """Open a new connection and get a client interface handle with the varlink methods installed.
 
         :param interface_name: an interface name, which the service this client object is
@@ -605,11 +589,12 @@ class Client(object):
 
         return self.handler(self._interfaces[interface_name], connection, namespaced=namespaced)
 
-    def open_connection(self):
+    def open_connection(self) -> socket.socket:
         """Open a new connection and return the socket.
         :exception OSError: anything socket.connect() throws
 
         """
+        assert self._socket_fn, "socket_fn not initialised"
         return self._socket_fn()
 
     def get_interfaces(self, socket_connection=None):
@@ -627,7 +612,7 @@ class Client(object):
         if close_socket:
             socket_connection.close()
 
-        return self.info['interfaces']
+        return self.info["interfaces"]
 
     def get_interface(self, interface_name, socket_connection=None):
         if not socket_connection:
@@ -639,7 +624,7 @@ class Client(object):
         _service = self.handler(self._interfaces["org.varlink.service"], socket_connection)
         # noinspection PyUnresolvedReferences
         desc = _service.GetInterfaceDescription(interface_name)
-        interface = Interface(desc['description'])
+        interface = Interface(desc["description"])
         self._interfaces[interface.name] = interface
 
         if close_socket:
@@ -647,7 +632,7 @@ class Client(object):
 
         return interface
 
-    def add_interface(self, interface):
+    def add_interface(self, interface: Interface) -> None:
         """Manually add or overwrite an interface definition from an Interface object.
 
         :param interface: an Interface() object
