@@ -1,43 +1,21 @@
-# coding=utf-8
-
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import inspect
 import json
 import os
 import socket
 import stat
-import string
-import sys
+from socketserver import BaseServer, StreamRequestHandler, ThreadingMixIn
 
-try:
-    from builtins import int
-    from builtins import object
-    from builtins import open
-    from builtins import range
-except ImportError:
-    pass
-
-from .error import (InterfaceNotFound, InvalidParameter, MethodNotImplemented, VarlinkEncoder, VarlinkError,
-                    ConnectionError)
+from .error import InterfaceNotFound, InvalidParameter, MethodNotImplemented, VarlinkEncoder, VarlinkError
 from .scanner import Interface
 
-try:
-    from socketserver import (StreamRequestHandler, BaseServer, ThreadingMixIn)
-except ImportError:  # Python2
-    from SocketServer import (StreamRequestHandler, BaseServer, ThreadingMixIn)
-
 if hasattr(os, "fork"):
-    try:
-        from socketserver import ForkingMixIn
-    except ImportError:  # Python2
-        from SocketServer import ForkingMixIn
+    from socketserver import ForkingMixIn
 
 from types import GeneratorType
+from typing import Optional, Union
 
 
-class Service(object):
+class Service:
     """Varlink service server handler
 
     To use the Service, a global object is instantiated:
@@ -73,10 +51,21 @@ class Service(object):
 
     """
 
-    def __init__(self, vendor='', product='', version='', url='', interface_dir='.', namespaced=False):
+    def __init__(
+        self,
+        vendor="",
+        product="",
+        version="",
+        url="",
+        interface_dir=".",
+        namespaced=False,
+        *,
+        json_encoder_cls=None,
+    ):
         """Initialize the service with the data org.varlink.service.GetInfo() returns
 
-        :param interface_dir: the directory with the \*.varlink files for the interfaces
+        :param interface_dir: the directory with the \\*.varlink files for the interfaces
+        :param json_encoder_cls: Optional; supply a json.JSONEncoder subclass to use instead of the default
 
         """
         self.vendor = vendor
@@ -85,34 +74,35 @@ class Service(object):
         self.url = url
         self.interface_dir = interface_dir
         self._namespaced = namespaced
+        self.json_encoder_cls = json_encoder_cls or VarlinkEncoder
 
         self.interfaces = {}
         self.interfaces_handlers = {}
         directory = os.path.dirname(__file__)
-        self._add_interface(os.path.abspath(os.path.join(directory, 'org.varlink.service.varlink')), self)
+        self._add_interface(os.path.abspath(os.path.join(directory, "org.varlink.service.varlink")), self)
 
     def GetInfo(self):
         """The standardized org.varlink.service.GetInfo() varlink method."""
         return {
-            'vendor': self.vendor,
-            'product': self.product,
-            'version': self.version,
-            'url': self.url,
-            'interfaces': list(self.interfaces.keys())
+            "vendor": self.vendor,
+            "product": self.product,
+            "version": self.version,
+            "url": self.url,
+            "interfaces": list(self.interfaces.keys()),
         }
 
-    def GetInterfaceDescription(self, interface):
+    def GetInterfaceDescription(self, interface: str) -> dict[str, str]:
         """The standardized org.varlink.service.GetInterfaceDescription() varlink method."""
         try:
             i = self.interfaces[interface]
         except KeyError:
             raise InterfaceNotFound(interface)
 
-        return {'description': i.description}
+        return {"description": i.description}
 
     def _handle(self, message, raw_message, _server=None, _request=None):
         try:
-            interface_name, _, method_name = message.get('method', '').rpartition('.')
+            interface_name, _, method_name = message.get("method", "").rpartition(".")
             if not interface_name or not method_name:
                 raise InterfaceNotFound(interface_name)
 
@@ -122,8 +112,8 @@ class Service(object):
 
             method = interface.get_method(method_name)
 
-            parameters = message.get('parameters', {})
-            if parameters == None:
+            parameters = message.get("parameters", {})
+            if parameters is None:
                 parameters = {}
 
             handler = self.interfaces_handlers[interface.name]
@@ -136,7 +126,9 @@ class Service(object):
                 if name not in parameters:
                     parameters[name] = None
 
-            parameters = interface.filter_params("server.call", method.in_type, self._namespaced, parameters, None)
+            parameters = interface.filter_params(
+                "server.call", method.in_type, self._namespaced, parameters, None
+            )
 
             func = getattr(handler, method_name, None)
 
@@ -145,43 +137,43 @@ class Service(object):
 
             kwargs = {}
 
-            if hasattr(inspect, "signature"):
-                sig = inspect.signature(func)
-                arg_names = [(sig.parameters[k].kind in (
-                    inspect.Parameter.POSITIONAL_OR_KEYWORD, inspect.Parameter.KEYWORD_ONLY) and k or None) for k in
-                             sig.parameters.keys()]
-            else:
-                from itertools import izip
-                spec = inspect.getargspec(func)
-                matched_args = [reversed(x) for x in [spec.args, spec.defaults or []]]
-                arg_names = dict(izip(*matched_args))
+            sig = inspect.signature(func)
+            arg_names = [
+                (
+                    sig.parameters[k].kind
+                    in (inspect.Parameter.POSITIONAL_OR_KEYWORD, inspect.Parameter.KEYWORD_ONLY)
+                    and k
+                    or None
+                )
+                for k in sig.parameters.keys()
+            ]
 
-            if message.get('more', False) or message.get('oneway', False) or message.get('upgrade', False):
-                if message.get('more', False) and '_more' in arg_names:
+            if message.get("more", False) or message.get("oneway", False) or message.get("upgrade", False):
+                if message.get("more", False) and "_more" in arg_names:
                     kwargs["_more"] = True
 
-                if message.get('oneway', False) and '_oneway' in arg_names:
+                if message.get("oneway", False) and "_oneway" in arg_names:
                     kwargs["_oneway"] = True
 
-                if message.get('upgrade', False) and '_upgrade' in arg_names:
+                if message.get("upgrade", False) and "_upgrade" in arg_names:
                     kwargs["_upgrade"] = True
 
-            if '_raw' in arg_names:
+            if "_raw" in arg_names:
                 kwargs["_raw"] = raw_message
-            if '_message' in arg_names:
+            if "_message" in arg_names:
                 kwargs["_message"] = message
-            if '_interface' in arg_names:
+            if "_interface" in arg_names:
                 kwargs["_interface"] = interface
-            if '_method' in arg_names:
+            if "_method" in arg_names:
                 kwargs["_method"] = method
-            if '_server' in arg_names:
+            if "_server" in arg_names:
                 kwargs["_server"] = _server
-            if '_request' in arg_names:
+            if "_request" in arg_names:
                 kwargs["_request"] = _request
 
             if self._namespaced:
                 # FIXME: check for Maybe before taking None as default value
-                out = func(*(getattr(parameters, k, default=None) for k in method.in_type.fields.keys()), **kwargs)
+                out = func(*(getattr(parameters, k, None) for k in method.in_type.fields.keys()), **kwargs)
             else:
                 # FIXME: check for Maybe before taking None as default value
                 out = func(*(parameters.get(k) for k in method.in_type.fields.keys()), **kwargs)
@@ -196,17 +188,31 @@ class Service(object):
                             continue
 
                         cont = True
-                        if '_continues' in o:
-                            cont = o['_continues']
-                            del o['_continues']
-                            yield {'continues': bool(cont),
-                                   'parameters': interface.filter_params("server.reply", method.out_type,
-                                                                         self._namespaced, o,
-                                                                         None) or {}}
+                        if "_continues" in o:
+                            cont = o["_continues"]
+                            del o["_continues"]
+                            yield {
+                                "continues": bool(cont),
+                                "parameters": interface.filter_params(
+                                    "server.reply",
+                                    method.out_type,
+                                    self._namespaced,
+                                    o,
+                                    None,
+                                )
+                                or {},
+                            }
                         else:
-                            yield {'parameters': interface.filter_params("server.reply", method.out_type,
-                                                                         self._namespaced, o,
-                                                                         None) or {}}
+                            yield {
+                                "parameters": interface.filter_params(
+                                    "server.reply",
+                                    method.out_type,
+                                    self._namespaced,
+                                    o,
+                                    None,
+                                )
+                                or {}
+                            }
 
                         if not cont:
                             return
@@ -216,10 +222,10 @@ class Service(object):
                     except StopIteration:
                         pass
             else:
-                if message.get('oneway', False):
+                if message.get("oneway", False):
                     yield None
                 else:
-                    yield {'parameters': out or {}}
+                    yield {"parameters": out or {}}
 
         except VarlinkError as error:
             yield error
@@ -238,13 +244,13 @@ class Service(object):
         if message[-1] == 0:
             message = message[:-1]
 
-        string = message.decode('utf-8')
+        string = message.decode("utf-8")
         handle = self._handle(json.loads(string), message, _server, _request)
         for out in handle:
-            if out == None:
+            if out is None:
                 return
             try:
-                yield json.dumps(out, cls=VarlinkEncoder).encode('utf-8')
+                yield json.dumps(out, cls=self.json_encoder_cls).encode("utf-8")
             except ConnectionError as e:
                 try:
                     handle.throw(e)
@@ -253,7 +259,7 @@ class Service(object):
 
     def _add_interface(self, filename, handler):
         if not os.path.isabs(filename):
-            filename = os.path.join(self.interface_dir, filename + '.varlink')
+            filename = os.path.join(self.interface_dir, filename + ".varlink")
 
         with open(filename) as f:
             interface = Interface(f.read())
@@ -261,7 +267,7 @@ class Service(object):
             self.interfaces_handlers[interface.name] = handler
 
     def _set_interface(self, filename, interface_class):
-        if 'class' in str(type(interface_class)):
+        if "class" in str(type(interface_class)):
             ic = interface_class
         else:
             ic = interface_class()
@@ -269,7 +275,6 @@ class Service(object):
         return interface_class
 
     def interface(self, filename):
-
         def decorator(interface_class):
             self._add_interface(filename, interface_class())
             return interface_class
@@ -277,21 +282,15 @@ class Service(object):
         return decorator
 
 
-def get_listen_fd():
+def get_listen_fd() -> Union[int, None]:
     if "LISTEN_FDS" not in os.environ:
         return None
     if "LISTEN_PID" not in os.environ:
         return None
-    try:
-        if int(os.environ["LISTEN_PID"]) != os.getpid():
-            return None
-    except:
+    if int(os.environ.get("LISTEN_PID", -1)) != os.getpid():
         return None
 
-    try:
-        fds = int(os.environ["LISTEN_FDS"])
-    except:
-        return None
+    fds = int(os.environ.get("LISTEN_FDS", -1))
 
     if fds < 1:
         return None
@@ -305,7 +304,7 @@ def get_listen_fd():
         except OSError:
             return None
 
-    fields = string.split(os.environ["LISTEN_FDNAMES"], ":")
+    fields = os.environ.get("LISTEN_FDNAMES", "").split(":")
 
     if len(fields) != fds:
         return None
@@ -320,6 +319,8 @@ def get_listen_fd():
             except OSError:
                 return None
 
+    return None
+
 
 class RequestHandler(StreamRequestHandler):
     """Varlink request handler
@@ -327,10 +328,11 @@ class RequestHandler(StreamRequestHandler):
     To use as an argument for the VarlinkServer constructor.
     Instantiate your own class and set the class variable service to your global :class:`Service` object.
     """
-    service = None
+
+    service: Optional[Service] = None
 
     def handle(self):
-        message = b''
+        message = b""
 
         self.request.setblocking(True)
         while not self.rfile.closed:
@@ -339,18 +341,18 @@ class RequestHandler(StreamRequestHandler):
             except BrokenPipeError:
                 break
 
-            if c == b'':
+            if c == b"":
                 break
 
-            if c != b'\0':
+            if c != b"\0":
                 message += c
                 continue
 
             for reply in self.service.handle(message, _server=self.server, _request=self.request):
-                if reply != None:
-                    self.wfile.write(reply + b'\0')
+                if reply is not None:
+                    self.wfile.write(reply + b"\0")
 
-            message = b''
+            message = b""
 
 
 class Server(BaseServer):
@@ -360,6 +362,7 @@ class Server(BaseServer):
 
         >>> import varlink
         >>> import os
+        >>> import sys
         >>>
         >>> service = varlink.Service(vendor='Example', product='Examples', version='1', url='http://example.com',
         >>>    interface_dir=os.path.dirname(__file__))
@@ -374,7 +377,7 @@ class Server(BaseServer):
         >>>
         >>> server = varlink.ThreadingServer(sys.argv[1][10:], ServiceRequestHandler)
         >>> server.serve_forever()
-    """
+    """  # noqa: E501
 
     address_family = socket.AF_INET
 
@@ -397,13 +400,13 @@ class Server(BaseServer):
         elif server_address.startswith("unix:"):
             self.address_family = socket.AF_UNIX
             address = server_address[5:]
-            m = address.rfind(';mode=')
+            m = address.rfind(";mode=")
             if m != -1:
-                self.mode = address[m + 6:]
+                self.mode = address[m + 6 :]
                 address = address[:m]
 
-            if address[0] == '@':
-                address = address.replace('@', '\0', 1)
+            if address[0] == "@":
+                address = address.replace("@", "\0", 1)
                 self.mode = None
             else:
                 self.remove_file = address
@@ -414,20 +417,28 @@ class Server(BaseServer):
 
         elif server_address.startswith("tcp:"):
             address = server_address[4:]
-            p = address.rfind(':')
+            p = address.rfind(":")
             if p != -1:
-                port = int(address[p + 1:])
+                port = int(address[p + 1 :])
                 address = address[:p]
             else:
-                raise ConnectionError("Invalid address 'tcp:%s'" % address)
-            address = address.replace('[', '')
-            address = address.replace(']', '')
+                raise ConnectionError(f"Invalid address 'tcp:{address}'")
+            address = address.replace("[", "")
+            address = address.replace("]", "")
 
             try:
-                res = socket.getaddrinfo(address, port, proto=socket.IPPROTO_TCP, flags=socket.AI_NUMERICHOST)
+                res = socket.getaddrinfo(
+                    address, port, proto=socket.IPPROTO_TCP, flags=socket.AI_NUMERICHOST
+                )
             except TypeError:
-                res = socket.getaddrinfo(address, port, self.address_family, self.socket_type, socket.IPPROTO_TCP,
-                                         socket.AI_NUMERICHOST)
+                res = socket.getaddrinfo(
+                    address,
+                    port,
+                    self.address_family,
+                    self.socket_type,
+                    socket.IPPROTO_TCP,
+                    socket.AI_NUMERICHOST,
+                )
 
             af, socktype, proto, canonname, sa = res[0]
             self.address_family = af
@@ -436,7 +447,7 @@ class Server(BaseServer):
             server_address = sa[0:2]
 
         else:
-            raise ConnectionError("Invalid address '%s'" % server_address)
+            raise ConnectionError(f"Invalid address '{server_address}'")
 
         BaseServer.__init__(self, server_address, RequestHandlerClass)
 
@@ -444,7 +455,7 @@ class Server(BaseServer):
             try:
                 self.server_bind()
                 self.server_activate()
-            except:
+            except Exception:  # TODO: maybe just OSError?
                 self.server_close()
                 raise
 
@@ -464,7 +475,7 @@ class Server(BaseServer):
         self.server_address = self.socket.getsockname()
 
         if self.server_address[0] == 0:
-            self.server_address = '@' + self.server_address[1:].decode('utf-8')
+            self.server_address = "@" + self.server_address[1:].decode("utf-8")
             if self.mode:
                 os.fchmod(self.socket.fileno(), mode=int(self.mode, 8))
         elif self.mode:
@@ -487,11 +498,11 @@ class Server(BaseServer):
         if self.remove_file:
             try:
                 os.remove(self.remove_file)
-            except:
+            except OSError:
                 pass
         self.socket.close()
 
-    def fileno(self):
+    def fileno(self) -> int:
         """Return socket file number.
 
         Interface required by selector.
@@ -514,7 +525,7 @@ class Server(BaseServer):
             # the socket and waits for GC to perform the actual close.
             request.shutdown(socket.SHUT_RDWR)
 
-        except:
+        except Exception:  # TODO: maybe just OSError?
             pass  # some platforms may raise ENOTCONN here
         self.close_request(request)
 
@@ -529,8 +540,11 @@ class Server(BaseServer):
         self.server_close()
 
 
-class ThreadingServer(ThreadingMixIn, Server): pass
+class ThreadingServer(ThreadingMixIn, Server):
+    pass
 
 
 if hasattr(os, "fork"):
-    class ForkingServer(ForkingMixIn, Server): pass
+
+    class ForkingServer(ForkingMixIn, Server):
+        pass
