@@ -1,8 +1,5 @@
 #!/usr/bin/env python
 
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import codecs
 import getopt
 import json
@@ -11,26 +8,20 @@ import os
 import shlex
 import socket
 import sys
+import textwrap
 import threading
 import time
 import unittest
 from sys import platform
 
-try:
-    from builtins import object
-    from builtins import range
-    from builtins import str
-except ImportError:
-    pass
-
 import varlink
 
-
 ######## CLIENT #############
 
+
 def run_client(client):
-    print('Connecting to %s\n' % client)
-    with client.open('org.varlink.certification') as con:
+    print(f"Connecting to {client}\n")
+    with client.open("org.varlink.certification") as con:
         ret = con.Start()
         client_id = ret["client_id"]
         print("client_id:", client_id)
@@ -66,11 +57,11 @@ def run_client(client):
 ######## SERVER #############
 
 service = varlink.Service(
-    vendor='Varlink',
-    product='Varlink Examples',
-    version='1',
-    url='http://varlink.org',
-    interface_dir=os.path.dirname(__file__)
+    vendor="Varlink",
+    product="Varlink Examples",
+    version="1",
+    url="http://varlink.org",
+    interface_dir=os.path.dirname(__file__),
 )
 
 
@@ -85,19 +76,22 @@ def sorted_json(dct):
 
 
 class CertificationError(varlink.VarlinkError):
-
     def __init__(self, wants, got):
-        varlink.VarlinkError.__init__(self,
-                                      {'error': 'org.varlink.certification.CertificationError',
-                                       'parameters': {'wants': wants, 'got': got}})
+        varlink.VarlinkError.__init__(
+            self,
+            {
+                "error": "org.varlink.certification.CertificationError",
+                "parameters": {"wants": wants, "got": got},
+            },
+        )
 
 
-@service.interface('org.varlink.certification')
-class CertService(object):
-    next_method = {}
+@service.interface("org.varlink.certification")
+class CertService:
+    next_method: dict[str, str] = {}
 
     def new_client_id(self, _server):
-        client_id = codecs.getencoder('hex')(os.urandom(16))[0].decode("ascii")
+        client_id = codecs.getencoder("hex")(os.urandom(16))[0].decode("ascii")
         if not hasattr(_server, "next_method"):
             _server.next_method = {}
         if not hasattr(_server, "lifetimes"):
@@ -127,42 +121,58 @@ class CertService(object):
     def assert_raw(self, client_id, _server, _raw, _message, wants):
         if wants != _message:
             del _server.next_method[client_id]
-            raise CertificationError(wants, json.loads(_raw.decode('utf-8')))
+            raise CertificationError(wants, json.loads(_raw.decode("utf-8")))
 
     def assert_cmp(self, client_id, _server, _raw, wants, _bool):
         if not _bool:
             del _server.next_method[client_id]
-            raise CertificationError(wants, json.loads(_raw.decode('utf-8')))
+            raise CertificationError(wants, json.loads(_raw.decode("utf-8")))
 
     def assert_method(self, client_id, _server, from_method, next_method):
         if not hasattr(_server, "next_method") or client_id not in _server.next_method:
-            raise CertificationError({"method": "org.varlink.certification.Start+++"},
-                                     {"method": "org.varlink.certification." + from_method})
+            raise CertificationError(
+                {"method": "org.varlink.certification.Start+++"},
+                {"method": "org.varlink.certification." + from_method},
+            )
 
         self.check_lifetimes(_server)
 
         if from_method != _server.next_method[client_id]:
-            raise CertificationError("Call to method org.varlink.certification." + _server.next_method[client_id],
-                                     "Call to method org.varlink.certification." + from_method)
+            raise CertificationError(
+                "Call to method org.varlink.certification." + _server.next_method[client_id],
+                "Call to method org.varlink.certification." + from_method,
+            )
         _server.next_method[client_id] = next_method
 
     def Start(self, _server=None, _raw=None, _message=None, _oneway=False):
         client_id = self.new_client_id(_server)
-        if 'parameters' in _message and not _message['parameters']:
-            del _message['parameters']
+        if "parameters" in _message and not _message["parameters"]:
+            del _message["parameters"]
         self.assert_method(client_id, _server, "Start", "Test01")
-        self.assert_raw(client_id, _server, _raw, _message, {
-            "method": "org.varlink.certification.Start",
-        })
+        self.assert_raw(
+            client_id,
+            _server,
+            _raw,
+            _message,
+            {
+                "method": "org.varlink.certification.Start",
+            },
+        )
         return {"client_id": client_id}
 
     # () -> (bool: bool)
     def Test01(self, client_id, _server=None, _raw=None, _message=None):
         self.assert_method(client_id, _server, "Test01", "Test02")
-        self.assert_raw(client_id, _server, _raw, _message, {
-            "method": "org.varlink.certification.Test01",
-            "parameters": {"client_id": client_id}
-        })
+        self.assert_raw(
+            client_id,
+            _server,
+            _raw,
+            _message,
+            {
+                "method": "org.varlink.certification.Test01",
+                "parameters": {"client_id": client_id},
+            },
+        )
         return {"bool": True}
 
     # (bool: bool) -> (int: int)
@@ -170,9 +180,9 @@ class CertService(object):
         self.assert_method(client_id, _server, "Test02", "Test03")
         wants = {
             "method": "org.varlink.certification.Test02",
-            "parameters": {"client_id": client_id, "bool": True}
+            "parameters": {"client_id": client_id, "bool": True},
         }
-        self.assert_cmp(client_id, _server, _raw, wants, _bool == True)
+        self.assert_cmp(client_id, _server, _raw, wants, _bool == True)  # noqa: E712
         self.assert_raw(client_id, _server, _raw, _message, wants)
         return {"int": 1}
 
@@ -181,7 +191,7 @@ class CertService(object):
         self.assert_method(client_id, _server, "Test03", "Test04")
         wants = {
             "method": "org.varlink.certification.Test03",
-            "parameters": {"client_id": client_id, "int": 1}
+            "parameters": {"client_id": client_id, "int": 1},
         }
         self.assert_cmp(client_id, _server, _raw, wants, _int == 1)
         self.assert_raw(client_id, _server, _raw, _message, wants)
@@ -192,7 +202,7 @@ class CertService(object):
         self.assert_method(client_id, _server, "Test04", "Test05")
         wants = {
             "method": "org.varlink.certification.Test04",
-            "parameters": {"client_id": client_id, "float": 1.0}
+            "parameters": {"client_id": client_id, "float": 1.0},
         }
         self.assert_cmp(client_id, _server, _raw, wants, _float == 1.0)
         self.assert_raw(client_id, _server, _raw, _message, wants)
@@ -203,7 +213,7 @@ class CertService(object):
         self.assert_method(client_id, _server, "Test05", "Test06")
         wants = {
             "method": "org.varlink.certification.Test05",
-            "parameters": {"client_id": client_id, "string": "ping"}
+            "parameters": {"client_id": client_id, "string": "ping"},
         }
         self.assert_cmp(client_id, _server, _raw, wants, _string == "ping")
         self.assert_raw(client_id, _server, _raw, _message, wants)
@@ -220,12 +230,12 @@ class CertService(object):
                 "bool": False,
                 "int": 2,
                 "float": math.pi,
-                "string": "a lot of string"
-            }
+                "string": "a lot of string",
+            },
         }
         self.assert_raw(client_id, _server, _raw, _message, wants)
         self.assert_cmp(client_id, _server, _raw, wants, _int == 2)
-        self.assert_cmp(client_id, _server, _raw, wants, _bool == False)
+        self.assert_cmp(client_id, _server, _raw, wants, _bool == False)  # noqa: E712
         self.assert_cmp(client_id, _server, _raw, wants, _float == math.pi)
         self.assert_cmp(client_id, _server, _raw, wants, _string == "a lot of string")
 
@@ -238,12 +248,12 @@ class CertService(object):
             "method": "org.varlink.certification.Test07",
             "parameters": {
                 "client_id": client_id,
-                "struct": {"bool": False, "int": 2, "float": math.pi, "string": "a lot of string"}
-            }
+                "struct": {"bool": False, "int": 2, "float": math.pi, "string": "a lot of string"},
+            },
         }
         self.assert_raw(client_id, _server, _raw, _message, wants)
         self.assert_cmp(client_id, _server, _raw, wants, _dict["int"] == 2)
-        self.assert_cmp(client_id, _server, _raw, wants, _dict["bool"] == False)
+        self.assert_cmp(client_id, _server, _raw, wants, _dict["bool"] == False)  # noqa: E712
         self.assert_cmp(client_id, _server, _raw, wants, _dict["float"] == math.pi)
         self.assert_cmp(client_id, _server, _raw, wants, _dict["string"] == "a lot of string")
         return {"map": {"foo": "Foo", "bar": "Bar"}}
@@ -251,11 +261,16 @@ class CertService(object):
     # (map: [string]string) -> (set: [string]())
     def Test08(self, client_id, _map, _server=None, _raw=None, _message=None):
         self.assert_method(client_id, _server, "Test08", "Test09")
-        self.assert_raw(client_id, _server, _raw, _message,
-                        {
-                            "method": "org.varlink.certification.Test08",
-                            "parameters": {"client_id": client_id, "map": {"foo": "Foo", "bar": "Bar"}}
-                        })
+        self.assert_raw(
+            client_id,
+            _server,
+            _raw,
+            _message,
+            {
+                "method": "org.varlink.certification.Test08",
+                "parameters": {"client_id": client_id, "map": {"foo": "Foo", "bar": "Bar"}},
+            },
+        )
         return {"set": {"one", "two", "three"}}
 
     # (set: [string]()) -> (mytype: MyType)
@@ -263,10 +278,7 @@ class CertService(object):
         self.assert_method(client_id, _server, "Test09", "Test10")
         wants = {
             "method": "org.varlink.certification.Test09",
-            "parameters": {
-                "client_id": client_id,
-                "set": {"one": {}, "three": {}, "two": {}}
-            }
+            "parameters": {"client_id": client_id, "set": {"one": {}, "three": {}, "two": {}}},
         }
         self.assert_raw(client_id, _server, _raw, _message, wants)
         self.assert_cmp(client_id, _server, _raw, wants, isinstance(_set, set))
@@ -275,10 +287,11 @@ class CertService(object):
         self.assert_cmp(client_id, _server, _raw, wants, "two" in _set)
         self.assert_cmp(client_id, _server, _raw, wants, "three" in _set)
         return {
-            "client_id": client_id,
             "mytype": {
-                "object": {"method": "org.varlink.certification.Test09",
-                           "parameters": {"map": {"foo": "Foo", "bar": "Bar"}}},
+                "object": {
+                    "method": "org.varlink.certification.Test09",
+                    "parameters": {"map": {"foo": "Foo", "bar": "Bar"}},
+                },
                 "enum": "two",
                 "struct": {"first": 1, "second": "2"},
                 "array": ["one", "two", "three"],
@@ -291,11 +304,11 @@ class CertService(object):
                         None,
                         {"foo": "foo", "bar": "bar"},
                         None,
-                        {"one": "foo", "two": "bar"}
+                        {"one": "foo", "two": "bar"},
                     ],
-                    "anon": {"foo": True, "bar": False}
-                }
-            }
+                    "anon": {"foo": True, "bar": False},
+                },
+            },
         }
 
     # method Test10(mytype: MyType) -> (string: string)
@@ -308,8 +321,10 @@ class CertService(object):
             "parameters": {
                 "client_id": client_id,
                 "mytype": {
-                    "object": {"method": "org.varlink.certification.Test09",
-                               "parameters": {"map": {"foo": "Foo", "bar": "Bar"}}},
+                    "object": {
+                        "method": "org.varlink.certification.Test09",
+                        "parameters": {"map": {"foo": "Foo", "bar": "Bar"}},
+                    },
                     "enum": "two",
                     "struct": {"first": 1, "second": "2"},
                     "array": ["one", "two", "three"],
@@ -320,26 +335,26 @@ class CertService(object):
                             None,
                             {"foo": "foo", "bar": "bar"},
                             None,
-                            {"one": "foo", "two": "bar"}
+                            {"one": "foo", "two": "bar"},
                         ],
-                        "anon": {"foo": True, "bar": False}
-                    }
-                }
-            }
+                        "anon": {"foo": True, "bar": False},
+                    },
+                },
+            },
         }
 
         if "nullable" in mytype:
-            self.assert_cmp(client_id, _server, _raw, wants, mytype["nullable"] == None)
+            self.assert_cmp(client_id, _server, _raw, wants, mytype["nullable"] is None)
             del mytype["nullable"]
 
         if "nullable_array_struct" in mytype:
-            self.assert_cmp(client_id, _server, _raw, wants, mytype["nullable_array_struct"] == None)
+            self.assert_cmp(client_id, _server, _raw, wants, mytype["nullable_array_struct"] is None)
             del mytype["nullable_array_struct"]
 
         self.assert_cmp(client_id, _server, _raw, wants, mytype == wants["parameters"]["mytype"])
 
         for i in range(1, 11):
-            yield {"string": "Reply number %d" % i, '_continues': i != 10}
+            yield {"string": f"Reply number {i}", "_continues": i != 10}
 
     # method Test11(last_more_replies: []string) -> ()
     def Test11(self, client_id, last_more_replies, _server=None, _raw=None, _message=None, _oneway=False):
@@ -350,26 +365,36 @@ class CertService(object):
             "parameters": {
                 "client_id": client_id,
                 "last_more_replies": [
-                    "Reply number 1", "Reply number 2", "Reply number 3", "Reply number 4",
-                    "Reply number 5", "Reply number 6", "Reply number 7", "Reply number 8",
-                    "Reply number 9", "Reply number 10"
-                ]
-            }
+                    "Reply number 1",
+                    "Reply number 2",
+                    "Reply number 3",
+                    "Reply number 4",
+                    "Reply number 5",
+                    "Reply number 6",
+                    "Reply number 7",
+                    "Reply number 8",
+                    "Reply number 9",
+                    "Reply number 10",
+                ],
+            },
         }
 
         self.assert_cmp(client_id, _server, _raw, wants, _oneway)
 
         for i in range(0, 10):
-            self.assert_cmp(client_id, _server, _raw, wants, last_more_replies[i] == "Reply number %d" % (i + 1))
+            self.assert_cmp(client_id, _server, _raw, wants, last_more_replies[i] == f"Reply number {i + 1}")
 
     # method End() -> ()
     def End(self, client_id, _server=None, _raw=None, _message=None):
         self.assert_method(client_id, _server, "End", "Start")
 
-        self.assert_raw(client_id, _server, _raw, _message, {
-            "method": "org.varlink.certification.End",
-            "parameters": {"client_id": client_id}
-        })
+        self.assert_raw(
+            client_id,
+            _server,
+            _raw,
+            _message,
+            {"method": "org.varlink.certification.End", "parameters": {"client_id": client_id}},
+        )
 
         del _server.next_method[client_id]
         return {"all_ok": True}
@@ -386,18 +411,29 @@ def run_server(address):
 
 ######## MAIN #############
 
+
 def usage():
-    print('Usage: %s [[--client] --varlink=<varlink address>]' % sys.argv[0], file=sys.stderr)
-    print('\tSelf Exec: $ %s' % sys.argv[0], file=sys.stderr)
-    print('\tServer   : $ %s --varlink=<varlink address>' % sys.argv[0], file=sys.stderr)
-    print('\tClient   : $ %s --client --varlink=<varlink address>' % sys.argv[0], file=sys.stderr)
-    print('\tClient   : $ %s --client --bridge=<bridge command>' % sys.argv[0], file=sys.stderr)
-    print('\tClient   : $ %s --client --activate=<activation command>' % sys.argv[0], file=sys.stderr)
+    arg0 = sys.argv[0]
+    print(
+        textwrap.dedent(
+            f"""\
+            Usage: {arg0} [[--client] --varlink=<varlink address>]
+            \tSelf Exec: $ {arg0}
+            \tServer   : $ {arg0} --varlink=<varlink address>
+            \tClient   : $ {arg0} --client --varlink=<varlink address>
+            \tClient   : $ {arg0} --client --bridge=<bridge command>
+            \tClient   : $ {arg0} --client --activate=<activation command>
+            """
+        ),
+        file=sys.stderr,
+    )
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     try:
-        opts, args = getopt.getopt(sys.argv[1:], "b:A:", ["help", "client", "varlink=", "bridge=", "activate="])
+        opts, args = getopt.getopt(
+            sys.argv[1:], "b:A:", ["help", "client", "varlink=", "bridge=", "activate="]
+        )
     except getopt.GetoptError:
         usage()
         sys.exit(2)
@@ -432,7 +468,7 @@ if __name__ == '__main__':
 
     if not address and not client_mode:
         if not hasattr(socket, "AF_UNIX"):
-            print("varlink activate: not supported on platform %s" % platform, file=sys.stderr)
+            print(f"varlink activate: not supported on platform {platform}", file=sys.stderr)
             usage()
             sys.exit(2)
 
@@ -440,6 +476,8 @@ if __name__ == '__main__':
         with varlink.Client.new_with_activate([__file__, "--varlink=$VARLINK_ADDRESS"]) as client:
             run_client(client)
     elif client_mode:
+        if client is None:
+            raise ValueError("--client requires at either of --varlink, --bridge or --activate")
         with client:
             run_client(client)
     else:
@@ -450,13 +488,14 @@ if __name__ == '__main__':
 
 ######## UNITTEST #############
 
+
 class TestService(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         if hasattr(socket, "AF_UNIX"):
-            cls.address = "unix:org.varlink.certification_" \
-                          + str(os.getpid()) \
-                          + threading.current_thread().name
+            cls.address = (
+                "unix:org.varlink.certification_" + str(os.getpid()) + threading.current_thread().name
+            )
         else:
             cls.address = "tcp:127.0.0.1:23456"
 
@@ -469,8 +508,7 @@ class TestService(unittest.TestCase):
         run_client(varlink.Client.new_with_address(self.address))
 
     def test_01(self):
-        with varlink.Client(self.address) as client, \
-                client.open('org.varlink.certification') as con:
+        with varlink.Client(self.address) as client, client.open("org.varlink.certification") as con:
             ret = con.Start()
             client_id = ret["client_id"]
             ret = con.Test01(client_id)
